import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.*;

public class retroWorld extends World
{
    MidiPlayer music;
    WavPlayer soundBite;
    WavPlayer musicWAV;
    
 
    Simon simon;
    Health health;
    GameOver gameover;
    Title title;
    StartButton startButton;
    InstructionsButton instructionsButton;
    CreditsButton creditsButton;
    CreditsScreen creditsScreen;
    PlayAgainButton playAgainButton;
        
    InstructionScreen instructionScreen;
    BackButton backButton;
    
    public static final int worldWidth = 600;
    public static final int worldHeight = 432;
    public static final int middle = worldWidth/2;
    private static final int scrollSpeed = 3;
    private static final int NUM_ENEMIES_FIRST_LEVEL = 20;
    private static final int NUM_ENEMIES_SECOND_LEVEL = 21;
    private static final int NUM_ENEMIES_FINAL_LEVEL = 2;
    
    private  GreenfootImage bg;
    private  GreenfootImage mask;
    private  GreenfootImage tempbg;
    boolean simonAlive = false;
    boolean isInstructionScreen = false;
    boolean isCreditsScreen = false;
    boolean isGameOverScreen = false;
   
   // private GreenfootImage level2;
   // private GreenfootImage level2Mask;
    
    private int scrollPos;
    private static int scrollEnd;
    
    private Color ground = Color.BLACK;
    private Color endLevel = Color.GREEN;
    private Color stairsRight = Color.RED;
    private Color stairsLeft = Color.BLUE;
    
    private Enemy enemyQueue[];

    private int index; //Index for current queued enemy
    
    private int level;

    /**
     * Constructor for retroWorld, instantiates all the objects.
     * 
     */
    public retroWorld()
    {    super(worldWidth, worldHeight, 1); 
        // Create a new world with 20x20 cells with a cell size of 10x10 pixels.
       reset();
       Greenfoot.start();
    } 
    
    
    
    public void reset(){
        
        // Create a new world with 20x20 cells with a cell size of 10x10 pixels.
        
      
        index=0;
        level=0;
         
        music = new MidiPlayer("daytime.mid");
        soundBite = new WavPlayer();
        musicWAV=new WavPlayer();
            
        // Instantiate our actors
        health = new Health();
        title = new Title();
        startButton = new StartButton();
        instructionsButton = new InstructionsButton();
        creditsButton = new CreditsButton();
        instructionScreen = new InstructionScreen();
        backButton = new BackButton();
        creditsScreen = new CreditsScreen();
        playAgainButton = new PlayAgainButton();
        
        bg = new GreenfootImage("CastlevaniaMapLevel01a.png");
        mask = new GreenfootImage("CastlevaniaMapLevel01aMask.png");
        
        tempbg = new GreenfootImage(worldWidth, worldHeight);
        simon = new Simon();
        simonAlive = false;
        
        scrollEnd= -bg.getWidth() + worldWidth;
        
        // Start the title screen when Run is pressed
        startTitleScreen();   
        
        
        
    }
    
    /** Displays the title screen and all the buttons
     * 
     */
    private void startTitleScreen()
    {
        addObject(title, 300, 216);
        addObject(startButton, 478, 296);
        addObject(instructionsButton, 474, 385);
        addObject(creditsButton, 105, 383);
        
    }
    
    /** Display our instruction screen when instructionsButton is pressed
     * 
     */
    private void displayInstructionScreen()
    {
        removeObject(title);
        removeObject(startButton);
        removeObject(instructionsButton);
        removeObject(creditsButton);
        addObject(instructionScreen, 300, 216);
        addObject(backButton, 507, 390);
    }
  
    /** Display our credits screen when creditsButton is pressed
     * 
     */
    private void displayCreditsScreen()
    {
        removeObject(title);
        removeObject(startButton);
        removeObject(instructionsButton);
        removeObject(creditsButton);
        addObject(creditsScreen, 300, 216);
        addObject(backButton, 301, 393);
    }
    
    /** Start the first level when the startButton is pressed.
     * 
     */
    private void startFirstLevel()
    {
        simonAlive = true;
        removeObject(title);
        removeObject(startButton);
        removeObject(instructionsButton);
        removeObject(creditsButton);
        level = 1;
        
        loadImage();
        addObject(simon, 150, 50);
        
        scrollPos = 0;
        scrollEnd = -bg.getWidth() + worldWidth;
        enemyQueue = new Enemy[NUM_ENEMIES_FIRST_LEVEL]; //Zombie Type 1, Werewolf 2, SeekingEye 3
       
        generateEnemiesFirstLevel();
      
        addObject(health, 100, 25);
        
        soundBite.open("chew05.wav");     
      
        soundBite.setLoops(1);
        soundBite.play();
        Greenfoot.delay(220);
        music.play();
    }
    
    
    /** Start the second level
     * 
     */
    private void startSecondLevel()
    {
       
        simon.dead();
        removeObject(simon);
        removeObject(health);
        level=2; 
       
        index=0;
        scrollPos=10;
        
        bg = new GreenfootImage("marioWorld.png");
        mask = new GreenfootImage("marioMask.png");
        scrollEnd = -bg.getWidth() + worldWidth;
        
        scroll(1);
        
        simon=new Simon();
         
        health = new Health();
        
        enemyQueue = new Enemy[NUM_ENEMIES_SECOND_LEVEL];
     
        generateEnemiesSecondLevel();
        
         
        addObject(health, 100, 25);
        addObject(simon, 150, 50);
        
        music.stop();
        music.load("smb1-Theme.mid");
        music.play();
        
        soundBite.open("getsome.wav");
        soundBite.setLoops(1);
        soundBite.play();
    }
    
    /** Start our boss level
     * 
     */
    public void startFinalLevel()
    {
        simon.dead();
        removeObject(simon);
        removeObject(health);
    
        level=3; 
       
        index=0;
        scrollPos=10;
        
        bg = new GreenfootImage("bossLevel.png");
        mask = new GreenfootImage("bossLevelMask.png");
        scrollEnd = -bg.getWidth() + worldWidth;
        
        scroll(1);
        
        simon=new Simon(); 
        health = new Health();
      
        enemyQueue = new Enemy[NUM_ENEMIES_FINAL_LEVEL];
     
        generateEnemiesFinalLevel();
          
        addObject(health, 100, 25);
        addObject(simon, 150, 50);
        
        music.stop();
        music.load("z2ow.mid");
        music.play();
        
        soundBite.open("ripem.wav");
        soundBite.setLoops(1);
        soundBite.play();
    }
    
    /** Our act method that checks and changes the display on the title screen or whether to load
     *  the next level when we reach the end of the current level
     */
    public void act()
    {   
        // Are we actually playing the game yet
        if(simonAlive){
        
            switch(level){
                case 1:
                    spawnFirstLevel();
                    break;
            
                 case 2:
                    spawnSecondLevel();
                    break;
             
                 case 3:
                    spawnFinalLevel();
                    break;
            
                 default:
                    break;
            
            }
            
            if (simonAlive && level==1 && endLevel.equals(mask.getColorAt(simon.getX() - scrollPos, simon.getY()))){
                startSecondLevel();
                return;
            }
            
            if (simonAlive && level==2 && endLevel.equals(mask.getColorAt(simon.getX() - scrollPos, simon.getY()))){
                startFinalLevel();
                return; 
            }
           
            //If statements to handle death by no hit points or falling into a hole
            if(health.getHitPoints() == 0){
                simonAlive=false;
                gameOver();
                return;
            }
           
            if (simon.getY() > worldHeight - 40){
                simonAlive = false;
                gameOver();
                return;
            }
        
        
        }//end if simon alive
        
        // Else we are at the title screen or game over screen
        else {
           if(level == 0 && isInstructionScreen == false && isCreditsScreen == false)
                startTitleScreen();
           
            if(level == 0 && Greenfoot.mousePressed(startButton))
            {
                startFirstLevel();
            }
                
            if(level == 0 && Greenfoot.mousePressed(instructionsButton)){
                displayInstructionScreen();
                isInstructionScreen = true;
            }
            
            if(level == 0 && Greenfoot.mousePressed(creditsButton)){
                displayCreditsScreen();
                isCreditsScreen = true;
            }
                
            if(level == 0 && Greenfoot.mousePressed(backButton) && isInstructionScreen == true)
            {
                removeObject(instructionScreen);
                removeObject(backButton);
                startTitleScreen();
                isInstructionScreen = false;
            }
            
            if(level == 0 && Greenfoot.mousePressed(backButton) && isCreditsScreen == true)
            {
                removeObject(creditsScreen);
                removeObject(backButton);
                startTitleScreen();
                isCreditsScreen = false;
            }
        }
        
        // HERE IS WHERE PROBLEM IS TO DELETE ALL OF THE STUFF FROM WORLD
        if(isGameOverScreen == true && Greenfoot.mousePressed(playAgainButton))
        {
               
             
                isGameOverScreen = false;
            
                List l=getObjects(Zombie.class);
                if(!l.isEmpty())
                for(int i=0;i<l.size();i++){
                  Zombie z= (Zombie) l.get(i);
                  z.removeFromWorld();
                    
                }
                
                l=getObjects(Werewolf.class);
                if(!l.isEmpty())
                for(int i=0;i<l.size();i++){
                  Werewolf w= (Werewolf) l.get(i);
                  w.removeFromWorld();
                    
                }
                
                l=getObjects(SeekingEye.class);
                if(!l.isEmpty())
                for(int i=0;i<l.size();i++){
                  SeekingEye s = (SeekingEye) l.get(i);
                  s.removeFromWorld();
                    
                }
                
                l=getObjects(Goomba.class);
                if(!l.isEmpty())
                for(int i=0;i<l.size();i++){
                  Goomba g = (Goomba) l.get(i);
                  g.removeFromWorld();
                    
                }
                
                l=getObjects(Turtle.class);
                if(!l.isEmpty())
                for(int i=0;i<l.size();i++){
                  Turtle t = (Turtle) l.get(i);
                  t.removeFromWorld();
                    
                }
                
                l=getObjects(Chris.class);
                if(!l.isEmpty())
                for(int i=0;i<l.size();i++){
                  Chris c = (Chris) l.get(i);
                  c.removeFromWorld();
                    
                }
                
                l=getObjects(Wily.class);
                if(!l.isEmpty())
                for(int i=0;i<l.size();i++){
                  Wily w = (Wily) l.get(i);
                  w.removeFromWorld();
                    
                }
                    
                removeObjects(getObjects(null));    
                reset();
        }       

    }
    
    /** Scroll the world image when Simon is in the center of the world
     * 
     */
    public boolean scrollWorld(int d)
    {
        if (d > 0 && scrollPos >= scrollEnd)
        {
     
            scroll(scrollSpeed);
            simon.cycleRight(scrollPos);
          
            return true;
        }
        else
            return false;
    }
    
    /** Update the scroll position relative to the background image width
     * 
     */
    private void setScrollPos(int x)
    {
        scrollPos -= x;
    }
    
    /** Load up the world image for initial use
     * 
     */
    private void loadImage()
    {
        tempbg.drawImage(bg, scrollPos, 0);
        setBackground(tempbg);
    }
    
    /** Scroll the world and update scroll position
     * 
     */
    private void scroll(int x)
    {
        tempbg.drawImage(bg, scrollPos, 0);
        setBackground(tempbg);
        setScrollPos(x);
    }

    /** Function to check if our Actor is on the ground
     * 
     */
    public boolean isGround(int xPos, int yPos)
    {
        if (xPos < 0 || xPos > worldWidth || 
            yPos < 0 || yPos > worldHeight)
            return false;
            
        if (ground.equals(mask.getColorAt(xPos - scrollPos, yPos)))
            return true;
        else
            return false;
    }

    /** Function to check if our Actor is on the Stairs
     * 
     */
    public boolean isStairs(int xPos, int yPos)
    {
        if (xPos < 0 || xPos > worldWidth || 
            yPos < 0 || yPos > worldHeight)
            return false;
            
        if (stairsRight.equals(mask.getColorAt(xPos - scrollPos, yPos)) ||  stairsLeft.equals(mask.getColorAt(xPos - scrollPos, yPos)))
            return true;
    
        return false;
    }
    
    /** Return 1 if our Actor is on the stairs that face right, -1 otherwise
     * 
     */
    public int getStairDirection(int xPos, int yPos)
    {
    if (stairsRight.equals(mask.getColorAt(xPos - scrollPos, yPos)))
            return 1;
        else
            return -1;
    }
    
    /** Return reference to simon to use in enemy classes
     * 
     */
    public Simon getSimon()
    {
        return simon;
    }
    
    /** Return regerence to health bar
     * 
     */
    public Health getHealth()
    {
        return health;
    }
    
    /** Return true if our Actor is not at either edge of the background image
     * 
     */
    public boolean canScrollWorld(int d)
    {
        if (d > 0 && scrollPos >= scrollEnd)
            return true;
        else
            return false;
    }
    
    /** Spawn enemies for the first (Castlevania) level out of the Enemy Queue
     * 
     */
    private void spawnFirstLevel()
    {
        if (index == NUM_ENEMIES_FIRST_LEVEL)
            return;
        
        if (Math.abs((enemyQueue[index].getPos() - scrollPos)) < 10)
            {
                switch(enemyQueue[index].getType())
                {
                    case 1:
                        Zombie z = new Zombie();
                        addObject(z, worldWidth, 50);
                        break;
                    
                    case 2:
                        Werewolf w = new Werewolf();
                        addObject(w, worldWidth, 50);
                        break;
                        
                    case 3:
                        SeekingEye e = new SeekingEye();
                        addObject(e, worldWidth, 50);
                        break;
                        
                    default:
                        break;
                }
                
                index++;
            }
    }
    
    /** Queue up Enemies for the first (Castlevania) level for spawning
     * 
     */
    private void generateEnemiesFirstLevel()
    {
        //Fill enemy queue
        for (int i = 0; i < NUM_ENEMIES_FIRST_LEVEL; i++)
        {
            Enemy e = new Enemy();
            enemyQueue[i] = e;
        }
        
         //Hard code enemy spawn locations and types
         enemyQueue[0].setType(1); //First Zombie
         enemyQueue[0].setPos(-400);
        
         enemyQueue[1].setType(1); //Second Zombie
         enemyQueue[1].setPos(-450);
        
         enemyQueue[2].setType(1); //Third Zombie
         enemyQueue[2].setPos(-500);
        
         enemyQueue[3].setType(3); //First SeekingEye
         enemyQueue[3].setPos(-1000);
         
         enemyQueue[4].setType(2); //First Werewolf
         enemyQueue[4].setPos(-1400);
        
         enemyQueue[5].setType(2); //second Werewolf
         enemyQueue[5].setPos(-2300);
        
         enemyQueue[6].setType(2); //third Werewolf
         enemyQueue[6].setPos(-2600);
        
         enemyQueue[7].setType(1); //4th Zombie
         enemyQueue[7].setPos(-2900);
        
         enemyQueue[8].setType(1); //5th Zombie
         enemyQueue[8].setPos(-3200);
        
         enemyQueue[9].setType(3); //second SeekingEye
         enemyQueue[9].setPos(-3200);
        
         enemyQueue[10].setType(1); //5th Zombie
         enemyQueue[10].setPos(-3400);
        
         enemyQueue[11].setType(2); //4th Werewolf
         enemyQueue[11].setPos(-3800);
        
         enemyQueue[12].setType(3); //third SeekingEye
         enemyQueue[12].setPos(-4000);
        
         enemyQueue[13].setType(1); //6th Zombie
         enemyQueue[13].setPos(-4600);
        
         enemyQueue[14].setType(2); //5th Werewolf
         enemyQueue[14].setPos(-4700);
        
         enemyQueue[15].setType(1); //7th Zombie
         enemyQueue[15].setPos(-4800);
        
         enemyQueue[16].setType(3); //4th SeekingEye
         enemyQueue[16].setPos(-5900);
        
         enemyQueue[17].setType(2); //6th Werewolf
         enemyQueue[17].setPos(-6400);
        
         enemyQueue[18].setType(2); //7th Werewolf
         enemyQueue[18].setPos(-7300);
        
         enemyQueue[19].setType(3); //5th SeekingEye
         enemyQueue[19].setPos(-7800);
    }
    
    /** Spawn enemies for the Second Level (Super Mario Bros.) out of the Enemy Queue
     * 
     */
    private void spawnSecondLevel()
    {
        if (index == NUM_ENEMIES_SECOND_LEVEL)
            return;
        
        if (Math.abs((enemyQueue[index].getPos() - scrollPos)) < 10)
            {
                switch(enemyQueue[index].getType())
                {
                    case 0:
                        Ship s = new Ship();
                        addObject(s, (worldWidth - 100), 50);
                        break;
                    
                    case 1:
                        Goomba g = new Goomba();
                        addObject(g, (worldWidth - 10), 50);
                        break;
                        
                    case 2:
                        Turtle t = new Turtle();
                        addObject(t, worldWidth, 50);
                        break;
                        
                    default:
                        break;
                }
                
                index++;
            }
    }
    
    /** Queue up enemies for the Second (Super Mario Bros.) level for spawning
     * 
     */
    private void generateEnemiesSecondLevel()
    {
        //Fill enemy queue
        for (int i = 0; i < NUM_ENEMIES_SECOND_LEVEL; i++)
        {
            Enemy e = new Enemy();
            enemyQueue[i] = e;
        }
        
      
        //Hard code enemy spawn locations and types
        //Exploding ship
        enemyQueue[0].setType(0);
        enemyQueue[0].setPos(-335);
        
        enemyQueue[1].setType(1); 
        enemyQueue[1].setPos(-550); 
        
        enemyQueue[2].setType(1); 
        enemyQueue[2].setPos(-800);
        
        enemyQueue[3].setType(1);
        enemyQueue[3].setPos(-950);
        
        enemyQueue[4].setType(2); 
        enemyQueue[4].setPos(-1100);
        
        enemyQueue[5].setType(2); 
        enemyQueue[5].setPos(-1300);
        
        enemyQueue[6].setType(2); 
        enemyQueue[6].setPos(-1550);
        
        enemyQueue[7].setType(2); 
        enemyQueue[7].setPos(-1550);
        
        enemyQueue[8].setType(1); 
        enemyQueue[8].setPos(-1850);
        
        enemyQueue[9].setType(2); 
        enemyQueue[9].setPos(-1950);
        
        enemyQueue[10].setType(1); 
        enemyQueue[10].setPos(-2250);
        
        enemyQueue[11].setType(1); 
        enemyQueue[11].setPos(-2400);
        
        enemyQueue[12].setType(2); 
        enemyQueue[12].setPos(-2600);
        
        enemyQueue[13].setType(2); 
        enemyQueue[13].setPos(-2850);
        
        enemyQueue[14].setType(1); 
        enemyQueue[14].setPos(-3150);
        
        enemyQueue[15].setType(1); 
        enemyQueue[15].setPos(-3250);
        
        enemyQueue[16].setType(1); 
        enemyQueue[16].setPos(-3250);
        
        enemyQueue[17].setType(2); 
        enemyQueue[17].setPos(-3800);
        
        enemyQueue[18].setType(2); 
        enemyQueue[18].setPos(-3900);
        
        enemyQueue[19].setType(1); 
        enemyQueue[19].setPos(-5100);
        
        enemyQueue[20].setType(1); 
        enemyQueue[20].setPos(-5250);
    }
    
    /** Spawn enemies for the Final (Zelda II) level from the Enemy Queue
     * 
     */
    private void spawnFinalLevel()
    {
        if (index == NUM_ENEMIES_FINAL_LEVEL)
            return;
        
        if (Math.abs((enemyQueue[index].getPos() - scrollPos)) < 10)
            {
                switch(enemyQueue[index].getType())
                {
                    case 0:
                        Ship s = new Ship();
                        addObject(s, (worldWidth - 100), 50);
                        break;
                    
                    case 1:
                        Wily w = new Wily();
                        addObject(w, (worldWidth - 50), 50);
                        music.stop();
                        music.close();
                        break;
                        
                    default:
                        break;
                }
                
                index++;
            }        
    }
    
    /** Queue up enemies for the Final (Zelda II) level for spawning
     * 
     */
    private void generateEnemiesFinalLevel()
    {
        //Fill enemy queue
        for (int i = 0; i < NUM_ENEMIES_FINAL_LEVEL; i++)
        {
            Enemy e = new Enemy();
            enemyQueue[i] = e;
        }        
      
        //Hard code enemy spawn locations and types
        //Exploding ship
        enemyQueue[0].setType(0);
        enemyQueue[0].setPos(-335);  
        
        enemyQueue[1].setType(1);
        enemyQueue[1].setPos(-2400);
    }
    
    /** Add game over screen
     * 
     */
    public void gameOver(){       
    
        gameover = new GameOver();
        music.stop();
        soundBite.open("GameOver.wav");
        soundBite.setLoops(1);
        soundBite.play();
        addObject(gameover,300,220);
        simon.dead();
        isGameOverScreen = true;
        addObject(playAgainButton, 304, 309);
    }    

    /** Play Chris Hansen soundbites and theme song for final battle
     * 
     */
    public void chrisSounds(){    
        music.stop();
        
        soundBite.open("chris.wav"); //"I'm Chris Hansen with dateline NBC" Goes here
        soundBite.setLoops(-1);
        soundBite.play();
 
        musicWAV.open("girls.wav"); //2 girls 1 cup theme
        musicWAV.setLoops(-1);
        musicWAV.play();
    }
    
    /** Return the current Scroll Position value
     * 
     */
    public int getScrollPos()
    {
        return scrollPos;
    }
    
    /** Return the current Level number
     * 
     */
    public int getLevel()
    {
        return level;
    }

    public void started()
    {

    }
    
    public void stopped()
    {
        music.stop();
        musicWAV.stop();
        soundBite.stop();     
        // music.close();
    }  
}